@@ -4,7 +4,7 @@
 *  Based upon CGI/Ex/Validate.pm v0.94 from Perl                     *
 *  For instructions on usage, see perldoc of CGI::Ex::Validate       *
 ***----------------------------------------------------------------**/
-// $Revision: 1.24 $
+// $Revision: 1.25 $
 
 function Validate () {
   this.error             = vob_error;
@@ -209,7 +209,7 @@ function vob_check_conditional (form, ifs, N_level, ifs_match) {
     /// get the field - allow for custom variables based upon a match
     var field = ref['field'];
     if (! field) return this.error("Missing field key during validate_if");
-    field = field.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+    field = field.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
       if (typeof(ifs_match) != 'object'
           || typeof(ifs_match[N]) == 'undefined') return ''
       return ifs_match[N];
@@ -747,7 +747,7 @@ function eob_as_hash (extra2) {
     if (! field) return alert("Missing field name");
     if (field_val['delegate_error']) {
       field = field_val['delegate_error'];
-      field = field.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+      field = field.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
         if (typeof(ifs_match) != 'object'
             || typeof(ifs_match[N]) == 'undefined') return ''
         return ifs_match[N];
@@ -789,7 +789,7 @@ function eob_get_error_text (err) {
   /// allow for delegated field names - only used for defaults
   if (field_val['delegate_error']) {
     field = field_val['delegate_error'];
-    field = field.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+    field = field.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
       if (typeof(ifs_match) != 'object'
           || typeof(ifs_match[N]) == 'undefined') return ''
       return ifs_match[N];
@@ -798,7 +798,7 @@ function eob_get_error_text (err) {
 
   /// the the name of this thing
   var name = (field_val['name']) ? field_val['name'] : "The field " +field;
-  name = name.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+  name = name.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
     if (typeof(ifs_match) != 'object'
         || typeof(ifs_match[N]) == 'undefined') return ''
     return ifs_match[N];
@@ -818,7 +818,7 @@ function eob_get_error_text (err) {
       if (extra[key]) ret = extra[key];
       else continue;
     }
-    ret = ret.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+    ret = ret.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
       if (typeof(ifs_match) != 'object'
           || typeof(ifs_match[N]) == 'undefined') return ''
       return ifs_match[N];
@@ -860,7 +860,7 @@ function eob_get_error_text (err) {
     var field2 = field_val["equals" + dig];
     var name2  = field_val["equals" +dig+ "_name"];
     if (! name2) name2 = "the field " +field2;
-    name2 = name2.replace(new RegExp('\\$(\\d+)','g'), function (N) {
+    name2 = name2.replace(new RegExp('\\$(\\d+)','g'), function (all, N) {
       if (typeof(ifs_match) != 'object'
           || typeof(ifs_match[N]) == 'undefined') return ''
       return ifs_match[N];
